from lxml import etree
from typing import Tuple


__all__ = ["validate", ]


class ValidationError(Exception):
    pass


def validate(xml: str, xsd: str) -> Tuple[bool, str]:
    parser = etree.XMLParser(ns_clean=True, recover=True, encoding='utf-8')
    # XSD
    xmlschema_doc = etree.fromstring(xsd.encode('utf-8'), parser=parser)
    xmlschema = etree.XMLSchema(xmlschema_doc)

    # XML
    xml_doc = etree.fromstring(xml.encode('utf-8'), parser=parser)
    result = xmlschema.validate(xml_doc)

    # xmlschema.error_log.last_error => lxml.etree._LogEntry
    return result, str(xmlschema.error_log.last_error)


# language=XSD
generate_invoice = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamla" xmlns:tns="http://www.szamlazz.hu/xmlszamla" elementFormDefault="qualified">

    <complexType name="vevoTipus">
        <sequence>
            <element name="nev" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="orszag" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="irsz" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="telepules" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="cim" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="email" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="sendEmail" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="adoalany" type="int" maxOccurs="1" minOccurs="0"></element>            <!-- Possible values for the "type" field: 7:business is based outside of the European Union, 6:business is based in the Europen Union, 1: has a hungarian tax number, 0: we don't know, if the buyer has a tax number, -1: no tax number -->
            <element name="adoszam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="adoszamEU" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="postazasiNev" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="postazasiOrszag" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="postazasiIrsz" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="postazasiTelepules" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="postazasiCim" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="vevoFokonyv" type="tns:vevoFokonyvTipus" maxOccurs="1" minOccurs="0"></element>                        
            <element name="azonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="alairoNeve" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="telefonszam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="megjegyzes" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="vevoFokonyvTipus">
        <sequence>
            <element name="konyvelesDatum" type="date" maxOccurs="1" minOccurs="0"></element>
            <element name="vevoAzonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="vevoFokonyviSzam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="folyamatosTelj" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="elszDatumTol" type="date" maxOccurs="1" minOccurs="0"></element>
            <element name="elszDatumIg" type="date" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="eladoTipus">
        <sequence>
            <element name="bank" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="bankszamlaszam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailReplyto" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailTargy" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailSzoveg" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="alairoNeve" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="eszamla" type="boolean" maxOccurs="1" minOccurs="1"></element>
            <element name="szamlaLetoltes" type="boolean" maxOccurs="1" minOccurs="1"></element>
            <element name="szamlaLetoltesPld" type="int" maxOccurs="1" minOccurs="0"></element>
            <element name="valaszVerzio" type="int" maxOccurs="1" minOccurs="0"></element>
            <element name="aggregator" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="guardian" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="cikkazoninvoice" type="boolean" maxOccurs="1" minOccurs="0"></element>

        </sequence>
    </complexType>

    <complexType name="tetelTipus">
        <sequence>
            <element name="megnevezes" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="azonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="mennyiseg" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="mennyisegiEgyseg" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="nettoEgysegar" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="afakulcs" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="arresAfaAlap" type="double" maxOccurs="1" minOccurs="0"></element>                        
            <element name="nettoErtek" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="afaErtek" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="bruttoErtek" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="megjegyzes" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="tetelFokonyv" type="tns:tetelFokonyvTipus" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="tetelekTipus">
        <sequence>
            <element name="tetel" type="tns:tetelTipus" maxOccurs="unbounded" minOccurs="1"></element>
        </sequence>
    </complexType>

    <complexType name="tetelFokonyvTipus">
        <sequence>
            <element name="gazdasagiEsem" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="gazdasagiEsemAfa" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="arbevetelFokonyviSzam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="afaFokonyviSzam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="elszDatumTol" type="date" maxOccurs="1" minOccurs="0"></element>
            <element name="elszDatumIg" type="date" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="fejlecTipus">
        <sequence>
            <element name="keltDatum" type="date" maxOccurs="1" minOccurs="0"></element>
            <element name="teljesitesDatum" type="date" maxOccurs="1" minOccurs="1"></element>
            <element name="fizetesiHataridoDatum" type="date" maxOccurs="1" minOccurs="1"></element>
            <element name="fizmod" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="penznem" type="string" maxOccurs="1"    minOccurs="1"></element>
            <element name="szamlaNyelve" type="tns:szamlaNyelveTipus" maxOccurs="1" minOccurs="1"></element>
            <element name="megjegyzes" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="arfolyamBank" type="string" maxOccurs="1" minOccurs="0"></element>   
            <element name="arfolyam" type="double" maxOccurs="1" minOccurs="0"></element>            <!-- If arfolyamBank='MNB' AND ther is no exchange rate, then the current exchange rate of MNB  will be used durring receipt creation (automatic MNB exchange rate import) -->
            <element name="rendelesSzam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="dijbekeroSzamlaszam" type="string" maxOccurs="1" minOccurs="0"></element> <!-- link to prepayment request -->
            <element name="elolegszamla" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="vegszamla" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="helyesbitoszamla" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="helyesbitettSzamlaszam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="dijbekero" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="szallitolevel" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="logoExtra" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaszamElotag" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="fizetendoKorrekcio" type="double" maxOccurs="1" minOccurs="0"></element>
            <element name="fizetve" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="arresAfa" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="eusAfa" type="boolean" maxOccurs="1" minOccurs="0"></element>            <!-- There is no hungarian VAT on the receipt. Data disclosure towards NTCAs Online Invoice System is not needed. -->
            <element name="szamlaSablon" type="string" maxOccurs="1" minOccurs="0"></element>       <!-- Codomain: 'SzlaMost' | 'SzlaAlap' | 'SzlaNoEnv' | 'Szla8cm' | 'SzlaTomb' | 'szlafuvarlevelesalap'-->
            <element name="elonezetpdf" type="boolean" maxOccurs="1" minOccurs="0"></element>       <!-- warrant preview pdf (no warrant is created) --> 
        </sequence>
    </complexType>

    <simpleType name="szamlaNyelveTipus">
        <restriction base="string">
            <enumeration value="hu"></enumeration>
            <enumeration value="en"></enumeration>
            <enumeration value="de"></enumeration>
            <enumeration value="it"></enumeration>
            <enumeration value="ro"></enumeration>
            <enumeration value="sk"></enumeration>
            <enumeration value="hr"></enumeration>
            <enumeration value="fr"></enumeration>
            <enumeration value="es"></enumeration>
            <enumeration value="cz"></enumeration>
            <enumeration value="pl"></enumeration>
        </restriction>
    </simpleType>

    <complexType name="transoflexTipus">
        <sequence>
            <!-- 5 digit number provided by TOF -->
            <element name="azonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="shipmentID" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="csomagszam" type="int" maxOccurs="1" minOccurs="0"></element>
            <element name="countryCode" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="zip" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="service" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="sprinterTipus">
        <sequence>
            <!-- 3-character abbreviation agreed with Sprinter -->
            <element name="azonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- 10-character long "feladókód" provided by Sprinter -->
            <element name="feladokod" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- "Iránykód" from Sprinter is Sprinters own special parcel shipment code, eg. "106" -->
            <element name="iranykod" type="string" maxOccurs="1" minOccurs="0"></element>
            <!--    Number of packages, this shows how many consignment note will be attached to the receipt-->
            <element name="csomagszam" type="int" maxOccurs="1" minOccurs="0"></element>
            <!-- unique per receipt, 7-13 characters long identifyer -->
            <element name="vonalkodPostfix" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- usually this is the 1 work day note -->
            <element name="szallitasiIdo" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="pppTipus">
        <sequence>
            <!-- 3-character abbreviation agreed with PPP -->
            <element name="vonalkodPrefix" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- unique per receipt, MAX 7 characters long identifyer -->
            <element name="vonalkodPostfix" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="mplTipus">
        <sequence>
            <!-- MPL customer code -->
            <element name="vevokod" type="string" maxOccurs="1" minOccurs="1"></element>
            <!-- Bar code is generated based on thes string -->
            <element name="vonalkod" type="string" maxOccurs="1" minOccurs="1"></element>
            <!-- Weight of the package, can contain a decimal point, if necessary -->
            <element name="tomeg" type="string" maxOccurs="1" minOccurs="1"></element>
            <!-- Optional configuration for special services icons, if not defined no icon will be shown -->
            <element name="kulonszolgaltatasok" type="string" maxOccurs="1" minOccurs="0"></element>
            <!--    Vallue ("erteknyilvanitas") field on the consignment note -->
            <element name="erteknyilvanitas" type="double" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <complexType name="fuvarlevelTipus">
        <sequence>
            <!-- TODO   the destination ("uticel") field is not used, should be removed after 2014.05.30, sprinter/iranykod field should be used instead  -->
            <element name="uticel" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- Codomain: TOF, PPP, SPRINTER, FOXPOST, EMPTY ,not necessary if there is no consignment note -->
            <element name="futarSzolgalat" type="string" maxOccurs="1" minOccurs="0"></element>
            <!--General bar code definition, this will be used, if no carrierspecific data is defined -->
            <element name="vonalkod" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- Comment on the consignment note -->
            <element name="megjegyzes" type="string" maxOccurs="1" minOccurs="0"></element>
            <!-- Trans-O-Flex node to upload -->
            <element name="tof" type="tns:transoflexTipus" maxOccurs="1" minOccurs="0"></element>
            <!-- PickPackPoint node to upload -->
            <element name="ppp" type="tns:pppTipus" maxOccurs="1" minOccurs="0"></element>
            <!-- Sprinter node to upload -->
            <element name="sprinter" type="tns:sprinterTipus" maxOccurs="1" minOccurs="0"></element>
            <!-- MPL node to upload-->
            <element name="mpl" type="tns:mplTipus" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>

    <element name="xmlszamla">
        <complexType>
            <sequence>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="elado" type="tns:eladoTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="vevo" type="tns:vevoTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fuvarlevel" type="tns:fuvarlevelTipus" maxOccurs="1" minOccurs="0"></element>
                <element name="tetelek" type="tns:tetelekTipus" maxOccurs="1" minOccurs="1"></element>
            </sequence>
        </complexType>
    </element>
</schema>
"""

# language=XSD
reverse_invoice = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamlast" xmlns:tns="http://www.szamlazz.hu/xmlszamlast" elementFormDefault="qualified">
    <complexType name="vevoTipus"><!-- If the TAX number of the buyer is missing from the original invoice, it can be added in this block -->
        <sequence>
            <element name="email" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="adoszam"   type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="adoszamEU" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <complexType name="eladoTipus">
        <sequence>
            <element name="emailReplyto" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailTargy" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailSzoveg" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="eszamla" type="boolean" maxOccurs="1" minOccurs="1"></element>
            <element name="szamlaLetoltes" type="boolean" maxOccurs="1" minOccurs="1"></element>
            <element name="szamlaLetoltesPld" type="int" maxOccurs="1" minOccurs="0"></element>
            <element name="aggregator" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="guardian" type="boolean" maxOccurs="1" minOccurs="0"></element>
            <element name="valaszVerzio" type="int" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <complexType name="fejlecTipus">
        <sequence>
            <element name="szamlaszam"      type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="keltDatum"       type="date"   maxOccurs="1" minOccurs="0"></element>
            <element name="teljesitesDatum" type="date"   maxOccurs="1" minOccurs="0"></element>
            <element name="tipus"           type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaSablon"    type="string" maxOccurs="1" minOccurs="0"></element> <!-- Értékkészlet: 'SzlaMost' | 'SzlaAlap' | 'SzlaNoEnv' | 'Szla8cm' | 'SzlaTomb' | 'SzlaFuvarlevelesAlap' -->
        </sequence>
    </complexType>

    <element name="xmlszamlast">
        <complexType>
            <sequence>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="elado" type="tns:eladoTipus" maxOccurs="1" minOccurs="0"></element>
                <element name="vevo" type="tns:vevoTipus" maxOccurs="1" minOccurs="0"></element>
            </sequence>
        </complexType>
    </element>
</schema>
"""

# language=XSD
credit_entry = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamlakifiz" xmlns:tns="http://www.szamlazz.hu/xmlszamlakifiz" elementFormDefault="qualified">
<complexType name="beallitasokTipus">
  <sequence>
    <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
    <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
    <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
    <element name="szamlaszam" type="string" maxOccurs="1" minOccurs="1"></element>
    <element name="additiv" type="boolean" maxOccurs="1" minOccurs="1"></element>
  </sequence>
</complexType>
<complexType name="kifizetesTipus">
  <sequence>
    <element name="datum" type="date" maxOccurs="1" minOccurs="1"></element>
    <element name="jogcim" type="string" maxOccurs="1" minOccurs="1"></element>
    <element name="osszeg" type="double" maxOccurs="1" minOccurs="1"></element>
    <element name="leiras" type="string" maxOccurs="1" minOccurs="0"></element>
  </sequence>
</complexType>
<complexType name="szamlaKifizTipus">
  <sequence>
    <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
    <element name="kifizetes" type="tns:kifizetesTipus" maxOccurs="5" minOccurs="0"></element>
  </sequence>
</complexType>
<element name="xmlszamlakifiz" type="tns:szamlaKifizTipus"></element>
</schema>"""


# language=XSD
query_invoice_pdf = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamlapdf"
xmlns:tns="http://www.szamlazz.hu/xmlszamlapdf" elementFormDefault="qualified">
  <complexType name="beallitasokTipus">
    <sequence>
      <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
      <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
      <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>      
      <element name="szamlaszam" type="string" maxOccurs="1" minOccurs="1"></element>
      <element name="valaszVerzio" type="int" maxOccurs="1" minOccurs="1"></element>
    </sequence>
  </complexType>
  <element name="xmlszamlapdf" type="tns:beallitasokTipus"></element>
</schema>
"""


# language=XSD
query_invoice_xml = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamlaxml" xmlns:tns="http://www.szamlazz.hu/xmlszamlaxml" elementFormDefault="qualified">
    <element name="xmlszamlaxml">
        <complexType>
            <sequence>
                <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
                <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
                <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
                <element name="szamlaszam" type="string" maxOccurs="1" minOccurs="0"></element>
                <element name="rendelesSzam" type="string" maxOccurs="1" minOccurs="0"></element>
                <element name="pdf" type="boolean" maxOccurs="1" minOccurs="0"></element>
            </sequence>
        </complexType>
    </element>
</schema>"""


# language=XML
delete_pro_forma_invoice = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlszamladbkdel" xmlns:tns="http://www.szamlazz.hu/xmlszamladbkdel" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <complexType name="fejlecTipus">
        <sequence>
            <element name="szamlaszam" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="rendelesszam" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <element name="xmlszamladbkdel">
      <complexType>
     <sequence>
        <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
        <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
     </sequence>
      </complexType>
    </element>
</schema>"""


# language=XML
generate_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlnyugtacreate" xmlns:tns="http://www.szamlazz.hu/xmlnyugtacreate" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <all>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="pdfLetoltes" type="boolean" maxOccurs="1" minOccurs="1"></element>
        </all>
    </complexType>
    <complexType name="fejlecTipus">
        <all>
            <element name="hivasAzonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="elotag" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="fizmod" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="penznem" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="devizaarf" type="double" maxOccurs="1" minOccurs="0"></element>
            <element name="devizabank" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="megjegyzes" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="pdfSablon" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="fokonyvVevo" type="string" maxOccurs="1" minOccurs="0"></element>
        </all>
    </complexType>
    <complexType name="tetelTipus">
        <all>
            <element name="megnevezes" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="azonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="mennyiseg" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="mennyisegiEgyseg" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="nettoEgysegar" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="afakulcs" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="netto" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="afa" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="brutto" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="fokonyv" type="tns:tetelFokonyvTipus" maxOccurs="1" minOccurs="0"></element>
        </all>
    </complexType>
    <complexType name="tetelFokonyvTipus">
        <all>
            <element name="arbevetel" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="afa" type="string" maxOccurs="1" minOccurs="0"></element>
        </all>
    </complexType>
    <complexType name="tetelekTipus">
        <sequence>
            <element name="tetel" type="tns:tetelTipus" maxOccurs="unbounded" minOccurs="1"></element>
        </sequence>
    </complexType>
    <!-- jóváírások -->
    <complexType name="kifizetesTipus">
        <all>
            <element name="fizetoeszkoz" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="osszeg" type="double" maxOccurs="1" minOccurs="1"></element>
            <element name="leiras" type="string" maxOccurs="1" minOccurs="0"></element>
        </all>
    </complexType>
    <complexType name="kifizetesekTipus">
        <sequence>
            <element name="kifizetes"             type="tns:kifizetesTipus"   maxOccurs="unbounded" minOccurs="1"></element>
        </sequence>
    </complexType>
    <element name="xmlnyugtacreate">
        <complexType>
            <all>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="tetelek" type="tns:tetelekTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="kifizetesek" type="tns:kifizetesekTipus" maxOccurs="1" minOccurs="0"></element>
            </all>
        </complexType>
    </element>
</schema>"""


# language=XML
reverse_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlnyugtast" xmlns:tns="http://www.szamlazz.hu/xmlnyugtast" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="pdfLetoltes" type="boolean" maxOccurs="1" minOccurs="1"></element>
        </sequence>
    </complexType>
    <complexType name="fejlecTipus">
        <sequence>
            <element name="nyugtaszam" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="pdfSablon" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="hivasAzonosito" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <element name="xmlnyugtast">
        <complexType>
            <sequence>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
            </sequence>
        </complexType>
    </element>
</schema>"""


# language=XML
query_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlnyugtaget" xmlns:tns="http://www.szamlazz.hu/xmlnyugtaget" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <all>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="pdfLetoltes" type="boolean" maxOccurs="1" minOccurs="1"></element>
        </all>
    </complexType>
    <complexType name="fejlecTipus">
        <all>
            <element name="nyugtaszam" type="string" maxOccurs="1" minOccurs="1"></element>
            <element name="hivasAzonosito" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="pdfSablon" type="string" maxOccurs="1" minOccurs="0"></element>
        </all>
    </complexType>
    <element name="xmlnyugtaget">
        <complexType>
            <all>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
            </all>
        </complexType>
    </element>
</schema>"""


# language=XML
send_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmlnyugtasend" xmlns:tns="http://www.szamlazz.hu/xmlnyugtasend" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <complexType name="fejlecTipus">
        <sequence>
            <element name="nyugtaszam" type="string" maxOccurs="1" minOccurs="1"></element>
        </sequence>
    </complexType>
    <complexType name="emailKuldes">
        <sequence>
            <element name="email" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailReplyto" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailTargy" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="emailSzoveg" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <element name="xmlnyugtasend">
        <complexType>
            <sequence>
                <element name="beallitasok" type="tns:beallitasokTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="fejlec" type="tns:fejlecTipus" maxOccurs="1" minOccurs="1"></element>
                <element name="emailKuldes" type="tns:emailKuldes" maxOccurs="1" minOccurs="0"></element>
            </sequence>
        </complexType>
    </element>
</schema>"""


# language=XML
tax_payer = """<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.szamlazz.hu/xmltaxpayer" xmlns:tns="http://www.szamlazz.hu/xmltaxpayer" elementFormDefault="qualified">
    <complexType name="beallitasokTipus">
        <sequence>
            <element name="felhasznalo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="jelszo" type="string" maxOccurs="1" minOccurs="0"></element>
            <element name="szamlaagentkulcs" type="string" maxOccurs="1" minOccurs="0"></element>
        </sequence>
    </complexType>
    <simpleType name="torszszamTipus">
        <restriction base="string">
            <length value="8" />
            <pattern value="[0-9]{8}" />
        </restriction>
    </simpleType>
    <element name="xmltaxpayer">
        <complexType>
            <sequence>
                <element name="beallitasok" type="tns:beallitasokTipus"maxOccurs="1" minOccurs="1"></element>
                <element name="torzsszam" type="tns:torszszamTipus"maxOccurs="1" minOccurs="1"></element>
            </sequence>
        </complexType>
    </element>
</schema>"""



import logging
from os import getenv
from pathlib import Path
from datetime import datetime

from szamlazz import SzamlazzClient, Header, Merchant, Buyer, PdfDataMissingError


logging.basicConfig(format='%(asctime)s %(name)s[:%(lineno)d] %(funcName)s %(levelname)s: %(message)s', datefmt='%b %d %H:%M:%S')
logging.root.setLevel(logging.DEBUG)
TODAY = datetime.today()
TMP_FOLDER_PDF_FILES = Path("./.tmp/pdf_files")


if __name__ == '__main__':
    TMP_FOLDER_PDF_FILES.mkdir(parents=True, exist_ok=True)
    # noinspection SpellCheckingInspection
    client = SzamlazzClient(
        agent_key=getenv('agent_key'),
        response_version=1,
    )

    demo_header = Header(
        invoice_number="E-DK-2021-15",
        creating_date=TODAY.strftime("%Y-%m-%d"),
        payment_date=TODAY.strftime("%Y-%m-%d"),

    )

    demo_merchant = Merchant(
        reply_email_address="merchant+noreply@demomerchant.hu",
        email_subject="Invoice notification",
        email_text="mail text"
    )

    demo_buyer = Buyer(
        email="buyer@example.com",
        tax_number="12345678-1-42",
        # tax_number_eu="HU55555555",
    )

    response = client.reverse_invoice(
        header=demo_header,
        merchant=demo_merchant,
        buyer=demo_buyer,
    )

    response.response.raise_for_status()  # check for HTTP transmission errors
    response.print_errors()  # check for szamlazz.py errors

    try:
        response.write_pdf_to_disk(TMP_FOLDER_PDF_FILES / f'{response.invoice_number}.STORNO.pdf')
    except PdfDataMissingError as e:
        raise e



__all__ = ["generate_invoice", "reverse_invoice", "credit_entry", "query_invoice_pdf", ]


# language=XML
generate_invoice: str = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamla xmlns="http://www.szamlazz.hu/xmlszamla"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           xsi:schemaLocation="http://www.szamlazz.hu/xmlszamla https://www.szamlazz.hu/szamla/docs/xsds/agent/xmlszamla.xsd">
    <beallitasok>
        <felhasznalo>{{ felhasznalo }}</felhasznalo>
        <jelszo>{{ jelszo }}</jelszo>
        <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
        <eszamla>{{ eszamla | lower }}</eszamla>
        <szamlaLetoltes>{{ szamlaLetoltes | lower }}</szamlaLetoltes>
        <valaszVerzio>{{ valaszVerzio }}</valaszVerzio>
        <aggregator>
        </aggregator>
    </beallitasok>
    <fejlec>
        <keltDatum>{{ header.creating_date }}</keltDatum>
        <teljesitesDatum>{{ header.payment_date }}</teljesitesDatum>
        <fizetesiHataridoDatum>{{ header.due_date }}</fizetesiHataridoDatum>
        <fizmod>{{ header.payment_type }}</fizmod>
        <penznem>{{ header.currency }}</penznem>
        <szamlaNyelve>{{ header.invoice_language }}</szamlaNyelve>
        <megjegyzes>{{ header.invoice_comment }}</megjegyzes>
        <arfolyamBank>{{ header.name_of_bank }}</arfolyamBank>
        <arfolyam>{{ header.exchange_rate }}</arfolyam>
        <rendelesSzam>{{ header.order_number }}</rendelesSzam>
        <dijbekeroSzamlaszam>{{ header.pro_forma_number_ref }}</dijbekeroSzamlaszam>
        <elolegszamla>{{ header.deposit_invoice | lower }}</elolegszamla>
        <vegszamla>{{ header.invoice_after_deposit_invoice | lower }}</vegszamla>
        <elolegSzamlaszam>{{ header.down_payment_invoice_number }}</elolegSzamlaszam>
        <helyesbitoszamla>{{ header.correction_invoice | lower }}</helyesbitoszamla>
        <helyesbitettSzamlaszam>{{ header.number_of_corrected_invoice }}</helyesbitettSzamlaszam>
        <dijbekero>{{ header.proforma_invoice | lower }}</dijbekero>
        <szamlaszamElotag>{{ header.invoice_prefix }}</szamlaszamElotag>
    </fejlec>
    <elado>
        <bank>{{ merchant.bank_name }}</bank>
        <bankszamlaszam>{{ merchant.bank_account_number }}</bankszamlaszam>
        <emailReplyto>{{ merchant.reply_email_address }}</emailReplyto>
        <emailTargy>{{ merchant.email_subject }}</emailTargy>
        <emailSzoveg>{{ merchant.email_text }}</emailSzoveg>
    </elado>
    <vevo>
        <nev>{{ buyer.name }}</nev>
        <orszag>{{ buyer.country }}</orszag>
        <irsz>{{ buyer.zip_code }}</irsz>
        <telepules>{{ buyer.city }}</telepules>
        <cim>{{ buyer.address }}</cim>
        <email>{{ buyer.email }}</email>
        <sendEmail>{{ buyer.send_email | lower }}</sendEmail>
        <adoalany>{{ buyer.tax_subject }}</adoalany>
        <adoszam>{{ buyer.tax_number }}</adoszam>
        <csoportazonosito>{{ buyer.group_id }}</csoportazonosito>
        <adoszamEU>{{ buyer.tax_number_eu }}</adoszamEU>
        <postazasiNev>{{ buyer.delivery_name }}</postazasiNev>
        <postazasiOrszag>{{ buyer.delivery_country }}</postazasiOrszag>
        <postazasiIrsz>{{ buyer.delivery_zip }}</postazasiIrsz>
        <postazasiTelepules>{{ buyer.delivery_city }}</postazasiTelepules>
        <postazasiCim>{{ buyer.delivery_address }}</postazasiCim>
        <azonosito>{{ buyer.identification }}</azonosito>
        <alairoNeve>{{ buyer.signatory_name }}</alairoNeve>
        <telefonszam>{{ buyer.phone_number }}</telefonszam>
        <megjegyzes>{{ buyer.comment }}</megjegyzes>
    </vevo>
    <fuvarlevel>
        <uticel> </uticel>
        <futarSzolgalat> </futarSzolgalat>
    </fuvarlevel>
    <tetelek>
    {%- for item in items %}
        <tetel>
            <megnevezes>{{ item.name }}</megnevezes>
            <mennyiseg>{{ item.quantity }}</mennyiseg>
            <mennyisegiEgyseg>{{ item.quantity_unit }}</mennyisegiEgyseg>
            <nettoEgysegar>{{ item.unit_price }}</nettoEgysegar>
            <afakulcs>{{ item.vat_rate }}</afakulcs>
            <nettoErtek>{{ item.net_price }}</nettoErtek>
            <afaErtek>{{ item.vat_amount }}</afaErtek>
            <bruttoErtek>{{ item.gross_amount }}</bruttoErtek>
            <megjegyzes>{{ item.comment_for_item }}</megjegyzes>
        </tetel>
    {% endfor -%}
    </tetelek>
</xmlszamla>
"""

# language=XML
reverse_invoice = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamlast xmlns="http://www.szamlazz.hu/xmlszamlast"
             xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
             xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlast https://www.szamlazz.hu/szamla/docs/xsds/agentst/xmlszamlast.xsd">
    <beallitasok>
        <felhasznalo>{{ felhasznalo }}</felhasznalo>
        <jelszo>{{ jelszo }}</jelszo>
        <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
        <eszamla>{{ eszamla | lower }}</eszamla>
        <szamlaLetoltes>{{ szamlaLetoltes | lower }}</szamlaLetoltes>
        <szamlaLetoltesPld>{{ szamlaLetoltesPld }}</szamlaLetoltesPld>
        <valaszVerzio>{{ valaszVerzio }}</valaszVerzio>
    </beallitasok>
    <fejlec>
        <szamlaszam>{{ header.invoice_number }}</szamlaszam>
        <keltDatum>{{ header.creating_date }}</keltDatum>
        <teljesitesDatum>{{ header.payment_date }}</teljesitesDatum>
        <tipus>SS</tipus>
        <szamlaSablon>{{ header.invoice_template }}</szamlaSablon>  <!-- Codomain: 'SzlaMost' | 'SzlaAlap' | 'SzlaNoEnv' | 'Szla8cm' | 'SzlaTomb' | 'SzlaFuvarlevelesAlap' -->
    </fejlec>
    <elado>
        <emailReplyto>{{ merchant.reply_email_address }}</emailReplyto>
        <emailTargy>{{ merchant.email_subject }}</emailTargy>
        <emailSzoveg>{{ merchant.email_text }}</emailSzoveg>
    </elado>
    <vevo>
        <email>{{ buyer.email }}</email>
        <adoszam>{{ buyer.tax_number }}</adoszam>
        <adoszamEU>{{ buyer.tax_number_eu }}</adoszamEU>
    </vevo>
</xmlszamlast>"""

# language=XML
credit_entry = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamlakifiz xmlns="http://www.szamlazz.hu/xmlszamlakifiz"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlakifiz https://www.szamlazz.hu/szamla/docs/xsds/agentkifiz/xmlszamlakifiz.xsd">
  <beallitasok> <!-- settings -->
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
    <szamlaszam>{{ szamlaszam }}</szamlaszam>
    <additiv>{{ additiv | lower }}</additiv>
  </beallitasok>
  {%- for disbursement in disbursements %}
  <kifizetes>
    <datum>{{ disbursement.date }}</datum>
    <jogcim>{{ disbursement.title }}</jogcim>
    <osszeg>{{ disbursement.amount }}</osszeg>
    <leiras>{{ disbursement.description }}</leiras>
  </kifizetes>
  {% endfor -%}
</xmlszamlakifiz>"""


# language=XML
query_invoice_pdf = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamlapdf xmlns="http://www.szamlazz.hu/xmlszamlapdf"
              xmlns:xsi="http://www.w3.org/2001/XMLSchemainstance"
              >
  <felhasznalo>{{ felhasznalo }}</felhasznalo>
  <jelszo>{{ jelszo }}</jelszo>
  <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
  <szamlaszam>{{ szamlaszam }}</szamlaszam>
  <valaszVerzio>{{ valaszVerzio }}</valaszVerzio>
</xmlszamlapdf>
"""


# language=XML
query_invoice_xml = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamlaxml xmlns="http://www.szamlazz.hu/xmlszamlaxml"
              xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
              xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlaxml https://www.szamlazz.hu/szamla/docs/xsds/agentxml/xmlszamlaxml.xsd">
  <felhasznalo>{{ felhasznalo }}</felhasznalo>
  <jelszo>{{ jelszo }}</jelszo>
  <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
  {% if rendelesSzam | length %}
  <rendelesSzam>{{ rendelesSzam }}</rendelesSzam>
  {% else %}
  <szamlaszam>{{ szamlaszam }}</szamlaszam>
  {% endif %}
  <pdf>{{ pdf | lower }}</pdf>
 </xmlszamlaxml>
"""

# language=XML
delete_pro_forma_invoice = """<?xml version="1.0" encoding="UTF-8"?>
<xmlszamladbkdel xmlns="http://www.szamlazz.hu/xmlszamladbkdel"
                 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                 xsi:schemaLocation="http://www.szamlazz.hu/xmlszamladbkdel http://www.szamlazz.hu/docs/xsds/szamladbkdel/xmlszamladbkdel.xsd">
  <beallitasok>
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
  </beallitasok>
  <fejlec>
    {% if rendelesszam | length %}
    <rendelesszam>{{ rendelesszam }}</rendelesszam>
    {% else %}
    <szamlaszam>{{ szamlaszam }}</szamlaszam>
    {% endif %}
  </fejlec>
</xmlszamladbkdel>"""


# language=XML
generate_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<xmlnyugtacreate xmlns="http://www.szamlazz.hu/xmlnyugtacreate" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtacreate http://www.szamlazz.hu/docs/xsds/nyugta/xmlnyugtacreate.xsd">
  <beallitasok>                                                      <!-- REQ         -->
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
    <pdfLetoltes>{{ pdfLetoltes }}</pdfLetoltes>
  </beallitasok>
  <fejlec>                                                <!-- REQ         -->
    <hivasAzonosito>{{ fejlec.hivasAzonosito }}</hivasAzonosito>     <!--     string  --> <!-- unique identifier of the call, duplication must be avoided-->
    <elotag>{{ fejlec.elotag }}</elotag>                    <!-- REQ string  --> <!-- receipt number prefix, required ==> NYGTA-2017-111 -->
    <fizmod>{{ fejlec.fizmod }}</fizmod>               <!-- REQ string  --> <!-- payment method, free text field, values ​​used on the interface are: átutalás, készpénz, bankkártya, csekk, utánvét, ajándékutalvány, barion, barter, csoportos beszedés, OTP Simple, kompenzáció, kupon, PayPal,PayU, SZÉP kártya, utalvány -->
    <penznem>{{ fejlec.penznem }}</penznem>                   <!-- REQ string  --> <!-- currency: Ft, HUF, EUR, USD stb. -->
    <devizabank>{{ fejlec.devizabank }}</devizabank>         <!--     string  --> <!-- in case of foreign bill (not Ft/HUF) the name of the Bank -->
    <devizaarf>{{ fejlec.devizaarf }}</devizaarf>                 <!--     string  --> <!-- exchange rate -->
    <megjegyzes>{{ fejlec.megjegyzes }}</megjegyzes>             <!--     string  --> <!-- free text description,  shown on the receipt -->
    <pdfSablon>{{ fejlec.pdfSablon }}</pdfSablon>                   <!--     string  --> <!--  in case of custom PDF template, the identifier of the used template-->
    <fokonyvVevo>{{ fejlec.fokonyvVevo }}</fokonyvVevo>                      <!--     string  --> <!-- general ledger ID of the customer -->
  </fejlec>
  <tetelek>
  {%- for tetel in tetelek %}
    <tetel>                                         <!-- REQ         --> <!-- at least one item is required to issue a receipt  -->
      <megnevezes>{{ tetel.megnevezes }}</megnevezes>         <!-- REQ string  --> <!-- name of the receipt -->
      <azonosito>{{ tetel.azonosito }}</azonosito>                                        <!--     string  --> <!-- ID of the receipt -->
      <mennyiseg>{{ tetel.mennyiseg }}</mennyiseg>                               <!-- REQ double  --> <!-- item quantity -->
      <mennyisegiEgyseg>{{ tetel.mennyisegiEgyseg }}</mennyisegiEgyseg>          <!-- REQ string  --> <!-- unit of quantity -->
      <nettoEgysegar>{{ tetel.nettoEgysegar }}</nettoEgysegar>                <!-- REQ double  --> <!-- net unit price -->
      <netto>{{ tetel.netto }}</netto>                                 <!-- REQ double  --> <!-- net value (quantity * net unit price) -->
      <afakulcs>{{ tetel.afakulcs }}</afakulcs>                                 <!-- REQ string  --> <!-- VAT rate, values: 0, 5, 10, 27, AAM, TAM, EU, EUK, MAA, F.AFA, K.AFA, ÁKK,HO, EUE, EUFADE, EUFAD37, ATK, NAM, EAM, KBAUK, KBAET -->
      <afa>{{ tetel.afa }}</afa>                                                <!-- REQ double  --> <!-- VAT total value -->
      <brutto>{{ tetel.brutto }}</brutto>                                      <!-- REQ double  --> <!-- gross total value -->
      <fokonyv>                                                             <!--             --> <!-- general ledger information -->
        <arbevetel>{{ tetel.fokonyv_arbevetel }}</arbevetel>                                   <!--     string  --> <!-- sales general ledger ID  -->
        <afa>{{ tetel.fokonyv_afa }}</afa>                                                     <!--     string  --> <!-- VAT general ledger ID -->
      </fokonyv>
    </tetel>
  {% endfor -%}
  </tetelek>
  <!--
    The <kifizetesek> section (payments) is not mandatory, but if present,
    then the sum of the values should be equal with the total amount of the receipt.
  -->
  {% if kifizetesek | length %}
  <kifizetesek>
  {%- for kifizetes in kifizetesek %}
    <kifizetes>
      <fizetoeszkoz>{{ kifizetes.fizetoeszkoz }}</fizetoeszkoz>
      <osszeg>{{ kifizetes.osszeg }}</osszeg>
      <leiras>{{ kifizetes.leiras }}</leiras>
    </kifizetes>
  {% endfor -%}
  </kifizetesek>
  {% endif %}
</xmlnyugtacreate>"""


# language=XML
reverse_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<xmlnyugtast xmlns="http://www.szamlazz.hu/xmlnyugtast"
             xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
             xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtast http://www.szamlazz.hu/docs/xsds/nyugtast/xmlnyugtast.xsd">
  <beallitasok>
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
    <pdfLetoltes>{{ pdfLetoltes }}</pdfLetoltes>
  </beallitasok> 
  <fejlec>
    <nyugtaszam>{{ nyugtaszam }}</nyugtaszam>
    {% if pdfSablon | length %}
    <pdfSablon>{{ pdfSablon }}</pdfSablon>
    {% endif %}
  </fejlec>
</xmlnyugtast>"""


# language=XML
query_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<xmlnyugtaget xmlns="http://www.szamlazz.hu/xmlnyugtaget"
              xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
              xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtaget http://www.szamlazz.hu/docs/xsds/nyugtaget/xmlnyugtaget.xsd">
  <beallitasok>
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
    <pdfLetoltes>{{ pdfLetoltes }}</pdfLetoltes>
  </beallitasok>
  <fejlec>
    <nyugtaszam>{{ nyugtaszam }}</nyugtaszam>
    {% if pdfSablon | length %}
    <pdfSablon>{{ pdfSablon }}</pdfSablon>
    {% endif %}
  </fejlec>
</xmlnyugtaget>"""


# language=XML
send_receipt = """<?xml version="1.0" encoding="UTF-8"?>
<xmlnyugtasend xmlns="http://www.szamlazz.hu/xmlnyugtasend"
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
               xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtasend http://www.szamlazz.hu/docs/xsds/nyugtasend/xmlnyugtasend.xsd">
  <beallitasok>
    <felhasznalo>{{ felhasznalo }}</felhasznalo>
    <jelszo>{{ jelszo }}</jelszo>
    <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
  </beallitasok>
  <fejlec>
    <nyugtaszam>{{ nyugtaszam }}</nyugtaszam>
  </fejlec>
  {% if sendAgainPreviousEmail %}
  <!-- e-mail details, if not defined, the previous e-mail will be sent  -->
  <emailKuldes>
    <email>{{ email_details.addresses }}</email>
    <emailReplyto>{{ email_details.reply_to_address }}</emailReplyto>
    <emailTargy>{{ email_details.subject }}</emailTargy>
    <emailSzoveg>{{ email_details.body_text }}</emailSzoveg>
  </emailKuldes>
  {% endif %}
</xmlnyugtasend>"""


# language=XML
tax_payer = """<?xml version="1.0" encoding="UTF-8"?>
<xmltaxpayer xmlns="http://www.szamlazz.hu/xmltaxpayer"
             xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
             xsi:schemaLocation="http://www.szamlazz.hu/xmltaxpayer http://www.szamlazz.hu/docs/xsds/agent/xmltaxpayer.xsd">
    <beallitasok>
        <felhasznalo>{{ felhasznalo }}</felhasznalo>
        <jelszo>{{ jelszo }}</jelszo>
        <szamlaagentkulcs>{{ szamlaagentkulcs }}</szamlaagentkulcs>
    </beallitasok>
    <torzsszam>{{ vat_number }}</torzsszam>
</xmltaxpayer>"""



import logging
from os import getenv
from pathlib import Path
from datetime import datetime

from szamlazz import SzamlazzClient, Disbursement, PdfDataMissingError


logging.basicConfig(format='%(asctime)s %(name)s[:%(lineno)d] %(funcName)s %(levelname)s: %(message)s', datefmt='%b %d %H:%M:%S')
logging.root.setLevel(logging.DEBUG)
TODAY = datetime.today()
TMP_FOLDER_PDF_FILES = Path("./.tmp/pdf_files")


if __name__ == '__main__':
    TMP_FOLDER_PDF_FILES.mkdir(parents=True, exist_ok=True)
    # noinspection SpellCheckingInspection
    client = SzamlazzClient(
        agent_key=getenv('agent_key'),
    )

    disbursement_1 = Disbursement(
        date=TODAY.strftime("%Y-%m-%d"),
        title="készpénz",
        amount=1337,
    )
    register_credit_entry_response = client.register_credit_entry(
        invoice_number="E-DK-2021-15",
        disbursements=[disbursement_1, ],
        additive=True,
    )

    register_credit_entry_response.response.raise_for_status()  # check for HTTP transmission errors
    register_credit_entry_response.print_errors()  # check for szamlazz.py errors

    try:
        register_credit_entry_response.write_pdf_to_disk(TMP_FOLDER_PDF_FILES / f'{register_credit_entry_response.invoice_number}.credit_entry.pdf')
    except PdfDataMissingError as e:
        # TODO: Do something here
        raise e



import logging
from os import getenv
from pathlib import Path
from datetime import datetime

from szamlazz import SzamlazzClient, Disbursement, PdfDataMissingError


logging.basicConfig(format='%(asctime)s %(name)s[:%(lineno)d] %(funcName)s %(levelname)s: %(message)s', datefmt='%b %d %H:%M:%S')
logging.root.setLevel(logging.DEBUG)
TODAY = datetime.today()
TMP_FOLDER_PDF_FILES = Path("./.tmp/pdf_files")


if __name__ == '__main__':
    TMP_FOLDER_PDF_FILES.mkdir(parents=True, exist_ok=True)
    # noinspection SpellCheckingInspection
    client = SzamlazzClient(
        agent_key=getenv('agent_key'),
    )

    action = client.delete_pro_forma_invoice(
        invoice_number="D-DK-3",
        # order_number="ASD-1",
    )

    action.response.raise_for_status()  # check for HTTP transmission errors
    action.print_errors()  # check for szamlazz.py errors

    if action.action_success:
        print("OK")
    else:
        print("NOK")



import logging
from os import getenv
from pathlib import Path
from datetime import datetime

from szamlazz import SzamlazzClient, Header, Merchant, Buyer, Item, PdfDataMissingError


logging.basicConfig(format='%(asctime)s %(name)s[:%(lineno)d] %(funcName)s %(levelname)s: %(message)s', datefmt='%b %d %H:%M:%S')
logging.root.setLevel(logging.DEBUG)
TODAY = datetime.today()
TMP_FOLDER_PDF_FILES = Path("./.tmp/pdf_files")


if __name__ == '__main__':
    TMP_FOLDER_PDF_FILES.mkdir(parents=True, exist_ok=True)
    # noinspection SpellCheckingInspection
    client = SzamlazzClient(
        agent_key=getenv('agent_key'),
    )

    demo_header = Header(
        creating_date=TODAY.strftime("%Y-%m-%d"),
        payment_date=TODAY.strftime("%Y-%m-%d"),
        due_date=TODAY.strftime("%Y-%m-%d"),
        payment_type="Átutalás",
        currency="HUF",
        invoice_language="hu",
        invoice_comment="No Comment",
        name_of_bank="MNB",
        exchange_rate=0.0,
        order_number=f'ORDER-{TODAY.strftime("%m%d")}',
        pro_forma_number_ref="",
        deposit_invoice=False,
        invoice_after_deposit_invoice=False,
        correction_invoice=False,
        number_of_corrected_invoice="",
        proforma_invoice=False,
        invoice_prefix="DK",
    )

    demo_merchant = Merchant(
        bank_name="OTP",
        bank_account_number="11111111-22222222-33333333",
        reply_email_address="merchant+noreply@demomerchant.hu",
        email_subject="Invoice notification",
        email_text="mail text"
    )

    demo_buyer = Buyer(
        name="Kovacs Bt.",
        zip_code="2030",
        city="Érd",
        address="Tárnoki út 23.",
        email="buyer@example.com",
        send_email=False,
        tax_number="12345678-1-42",
        delivery_name="Kovács Bt. mailing name",
        delivery_zip="2040",
        delivery_city="Budaörs",
        delivery_address="Szivárvány utca 8.",
        identification="1234",
        phone_number="Tel:+3630-555-55-55, Fax:+3623-555-555",
        comment="Call extension 214 from the reception",
    )

    item_1 = Item(
        name="Eladó izé 1 X",
        quantity="2.0",
        quantity_unit="db",
        unit_price="10000",
        vat_rate="27",
        net_price="20000.0",
        vat_amount="5400.0",
        gross_amount="25400.0",
        comment_for_item="lorem ipsum 2",
    )

    item_2 = Item(
        name="Eladó izé 2 X",
        quantity="2.0",
        quantity_unit="db",
        unit_price="10000",
        vat_rate="27",
        net_price="20000.0",
        vat_amount="5400.0",
        gross_amount="25400.0",
        comment_for_item="lorem ipsum 2X",
    )

    item_3 = Item(
        name="Eladó izé 3 X",
        quantity="2.0",
        quantity_unit="db",
        unit_price="10000",
        vat_rate="27",
        net_price="20000.0",
        vat_amount="5400.0",
        gross_amount="25400.0",
        comment_for_item="lorem ipsum 2X",
    )

    invoice_response = client.generate_invoice(
        header=demo_header,
        merchant=demo_merchant,
        buyer=demo_buyer,
        items=[item_1, item_2, item_3, ],
    )

    invoice_response.response.raise_for_status()  # check for HTTP transmission errors
    invoice_response.print_errors()  # check for szamlazz.py errors

    try:
        invoice_response.write_pdf_to_disk(TMP_FOLDER_PDF_FILES / f'{invoice_response.invoice_number}.invoice.pdf')
    except PdfDataMissingError as e:
        # TODO: Do something here
        raise e



import logging
from typing import List, Tuple

import requests
from jinja2 import Template
from requests.models import Response

from szamlazz import templates
from szamlazz import xsd
from szamlazz.models import Header, Merchant, Buyer, Item, Disbursement, SzamlazzResponse, EmailDetails

__all__ = ["SzamlazzClient", ]
logger = logging.getLogger(__name__)


class SzamlazzClient:
    url = "https://www.szamlazz.hu/szamla/"

    def __init__(self,
                 username: str = "",
                 password: str = "",
                 agent_key: str = "",
                 response_version: int = 2,
                 ):
        """
        :param username: Számlázz.hu user
        :param password: Számlázz.hu’s user password
        :param agent_key: Számlázz.hu / Számla Agent Kulcs (key)
        :param response_version: Text|PDF or XML+PDF answer.

        response_version options:
            - 1: gives a simple text or PDF as answer.
            - 2: xml answer, in case you asked for the PDF as well, it will be included in the XML with base64 coding.
        """
        self.username = username
        self.password = password
        self.agent_key = agent_key
        self.response_version = response_version

        # authentication guards
        if all(v == "" for v in [self.username, self.password, self.agent_key]):
            raise AssertionError("A username/password combination OR the Agent Key (Számla Agent Kulcs) must be provided during initialisation")
        if all(v != "" for v in [self.username, self.password, self.agent_key]):
            raise AssertionError("Only one authentication method is allowed")

    @property
    def can_extract_pdf(self):
        return True if self.response_version == 2 else False

    def generate_invoice(self,
                         header: Header,
                         merchant: Merchant,
                         buyer: Buyer,
                         items: List[Item],
                         e_invoice: bool = True,
                         invoice_download: bool = True,
                         ) -> SzamlazzResponse:
        """
        https://docs.szamlazz.hu/#generating-invoices
        :param header: Header
        :param merchant: Merchant
        :param buyer: Buyer
        :param items: List[Item]
        :param e_invoice: bool (default=True)
        :param invoice_download: bool (default=True)
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        settings["eszamla"] = e_invoice
        settings["szamlaLetoltes"] = invoice_download
        payload_xml = {
            "header": header,
            "merchant": merchant,
            "buyer": buyer,
            "items": items,
            **settings,  # see SzamlazzClient.get_basic_settings() for details
        }
        r = self.request_maker(
            action="action-xmlagentxmlfile",
            template=templates.generate_invoice,
            template_data=payload_xml,
            xsd_xml=xsd.generate_invoice,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlszamlavalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def reverse_invoice(self,
                        header: Header,
                        merchant: Merchant,
                        buyer: Buyer,
                        e_invoice: bool = True,
                        invoice_download: bool = True,  # True = Generated PDF will be returned
                        invoice_download_copy: int = 1,  # 1=PDF copy | 2=original
                        ) -> SzamlazzResponse:
        """
        Reversing an invoice (storno)

        In case you request the creation of a paper-based (non-electronic) invoice and you've set `invoice_download` to `True`,
        you can indicate that you not only need the original version of the invoice but the copy in a single PDF as well.
        You can indicate this in the `invoice_download_copy` parameter:
          * 1: PDF copy
          * 2: original
        :param header: Header
        :param merchant: Merchant
        :param buyer: Buyer
        :param e_invoice: True if E-Invoice
        :param invoice_download: True to retrieve the generated PDF
        :param invoice_download_copy: 1 = PDF copy | 2 = Original PDF
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        settings["eszamla"] = e_invoice
        settings["szamlaLetoltes"] = invoice_download
        settings["szamlaLetoltesPld"] = invoice_download_copy
        payload_xml = {
            "header": header,
            "merchant": merchant,
            "buyer": buyer,
            **settings,  # see SzamlazzClient.get_basic_settings() for details
        }
        r = self.request_maker(
            action="action-szamla_agent_st",
            template=templates.reverse_invoice,
            template_data=payload_xml,
            xsd_xml=xsd.reverse_invoice,
        )
        response = SzamlazzResponse(r, xml_namespace="")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def register_credit_entry(self,
                              invoice_number: str,
                              disbursements: List[Disbursement],
                              additive: bool = False,
                              ) -> SzamlazzResponse:
        """
        Registering a credit entry.

        :raise ValueError: On more than 5 credit entries in disbursements: List[Disbursement]
        :param invoice_number: szamlaszam
        :param disbursements: List[Disbursement]
        :param additive: if it is True, then the former credit entries will be retained [default=False]
        :return: SzamlazzResponse
        """
        if len(disbursements) > 5:
            raise ValueError("You can register maximum of 5 credit entries")

        settings = self.get_basic_settings()
        settings["szamlaszam"] = invoice_number
        settings["additiv"] = additive
        payload_xml = {
            "disbursements": disbursements,
            **settings,  # see SzamlazzClient.get_basic_settings() for details
        }
        r = self.request_maker(
            action="action-szamla_agent_kifiz",
            template=templates.credit_entry,
            template_data=payload_xml,
            xsd_xml=xsd.credit_entry,
        )
        response = SzamlazzResponse(r, xml_namespace="")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def query_invoice_pdf(self,
                          invoice_number: str,
                          ) -> SzamlazzResponse:
        """
        There are two different types of the requested pdf:
          * in the case of a regular, paper-based invoice, it contains the next copy, or it can be 1 or 2 copies in the case it has not been printed before, depending on the settings.
          * in the case of an e-invoice, there is no such thing as copies, so it will give the original version back.

        The type of response depends on the value of the <valaszVerzio> | <response_version> variable sent in the request.
          * if it is 1 or not set, the response will be a PDF file
          * if it is 2, the response will be an XML file

        :param invoice_number: szamlaszam
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        settings["szamlaszam"] = invoice_number
        r = self.request_maker(
            action="action-szamla_agent_pdf",
            template=templates.query_invoice_pdf,
            template_data=settings,
            xsd_xml=xsd.query_invoice_pdf,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlszamlavalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        return response

    def query_invoice_xml(self,
                          invoice_number: str = "",
                          order_number: str = "",
                          pdf: bool = True,
                          ) -> SzamlazzResponse:
        """
        Order number can be used in the query. In this case the last receipt with this order number will be returned

        :param invoice_number: szamlaszam
        :param order_number: rendelesSzam
        :param pdf: pdf
        :return: SzamlazzResponse
        """
        if invoice_number == "" and order_number == "":
            raise AssertionError("invoice_number OR order_number must be provided")

        settings = self.get_basic_settings()
        settings["szamlaszam"] = invoice_number
        settings["rendelesSzam"] = order_number
        settings["pdf"] = pdf
        r = self.request_maker(
            action="action-szamla_agent_xml",
            template=templates.query_invoice_xml,
            template_data=settings,
            xsd_xml=xsd.query_invoice_xml

        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/szamla}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        return response

    def delete_pro_forma_invoice(self,
                                 invoice_number: str = "",
                                 order_number: str = "",
                                 ) -> SzamlazzResponse:
        """
        Order number can be used in the query. In this case the last receipt with this order number will be returned

        https://docs.szamlazz.hu/#deleting-a-pro-forma-invoice

        :param invoice_number: szamlaszam - pro forma invoice's unique number
        :param order_number: rendelesszam - manually added to the pro forma invoice upon generation
        :return: SzamlazzResponse
        """
        if invoice_number == "" and order_number == "":
            raise AssertionError("invoice_number OR order_number must be provided")

        settings = self.get_basic_settings()
        settings["szamlaszam"] = invoice_number
        settings["rendelesszam"] = order_number
        r = self.request_maker(
            action="action-szamla_agent_dijbekero_torlese",
            template=templates.delete_pro_forma_invoice,
            template_data=settings,
            xsd_xml=xsd.delete_pro_forma_invoice,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlszamladbkdelvalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        return response

    def generate_receipt(self, payload: dict) -> Response:
        """
        https://docs.szamlazz.hu/#generating-a-receipt

        Key details from https://docs.szamlazz.hu/#expected-results-of-generating-receipts
        Every successful call contains the <sikeres>true</sikeres> field in the response.
        In case the generation of the receipt fails and the result field is false: <sikeres>false</sikeres>,
        two additional fields will be included in the response, <hibakod/> for the error code and <hibauzenet/> for the error message itself.

        Note: You should use hivasAzonosito to make the call fault tolerant.
              If this field is in use, it needs to be unique, otherwise the API call will be unsuccessful.
              This ensures that if the same XML is posted multiple times, it will not duplicate an existing receipt.

        :payload: dict
        :return: requests.models.Response
        """

        # language=Python
        """
from szamlazz import SzamlazzClient

client = SzamlazzClient(
agent_key="ASD123",
)
settings = client.get_basic_settings()
fejlec = {
    "hivasAzonosito": "",
    "elotag": "",
    "fizmod": "",
    "penznem": "",
    "devizabank": "",
    "devizaarf": "",
    "megjegyzes": "",
    "pdfSablon": "",
    "fokonyvVevo": "",
}
tetel_1 = {
    "megnevezes": "",
    "azonosito": "",
    "mennyiseg": "",
    "mennyisegiEgyseg": "",
    "nettoEgysegar": "",
    "netto": "",
    "afakulcs": "",
    "afa": "",
    "brutto": "",
    "fokonyv_arbevetel": "",
    "fokonyv_afa": "",
}
payload = {
    "fejlec": fejlec,
    "tetelek": [tetel_1, ],
    **settings
}

# pass payload to generate_receipt(payload=payload)
        """

        return self.request_maker(
            action="action-szamla_agent_nyugta_create",
            template=templates.generate_receipt,
            template_data=payload,
            xsd_xml=xsd.generate_receipt,
        )

    def reverse_receipt(self,
                        receipt_number: str,
                        pdf_template: str = "",
                        ) -> SzamlazzResponse:
        """
        https://docs.szamlazz.hu/#reversing-a-receipt-storno
        :param receipt_number: [string] <nyugtaszam>
        :param pdf_template: <pdfSablon>
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        settings["nyugtaszam"] = receipt_number
        settings["pdfSablon"] = pdf_template
        r = self.request_maker(
            action="action-szamla_agent_nyugta_storno",
            template=templates.reverse_receipt,
            template_data=settings,
            xsd_xml=xsd.reverse_receipt,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlnyugtavalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def query_receipt(self,
                      receipt_number: str,
                      pdf_template: str = "",
                      ) -> SzamlazzResponse:
        """
        https://docs.szamlazz.hu/#querying-a-receipt
        :param receipt_number: [string] <nyugtaszam>
        :param pdf_template: <pdfSablon>
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        settings["nyugtaszam"] = receipt_number
        settings["pdfSablon"] = pdf_template
        r = self.request_maker(
            action="action-szamla_agent_nyugta_get",
            template=templates.query_receipt,
            template_data=settings,
            xsd_xml=xsd.query_receipt,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlnyugtavalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def send_receipt(self,
                     email_details: EmailDetails,
                     send_again_previous_email: bool = False,
                     ) -> SzamlazzResponse:
        """
        https://docs.szamlazz.hu/#sending-a-receipt

        In case you would like to provide multiple e-mail addresses in the e-mail field of the XML file, they must be separated by “,” (comma).
        Example:
        email_details.addresses = "abc@domain.com,def@domain.com,ghi@domain.com"

        :param email_details: [EmailDetails]
        :param send_again_previous_email: if True, the previous e-mail will be sent
        :return: SzamlazzResponse
        """
        settings = self.get_basic_settings()
        payload = {
            "email_details": email_details,
            "sendAgainPreviousEmail": send_again_previous_email,
            **settings,
        }
        r = self.request_maker(
            action="action-szamla_agent_nyugta_send",
            template=templates.send_receipt,
            template_data=payload,
            xsd_xml=xsd.send_receipt,
        )
        response = SzamlazzResponse(r, xml_namespace="{http://www.szamlazz.hu/xmlnyugtasendvalasz}")
        logger.info(f"success = {response.http_request_success}")
        logger.info(f"invoice_number = {response.invoice_number}")
        logger.info(f"buyer_account_url = {response.buyer_account_url}")
        return response

    def query_taxpayer(self,
                       vat_number: str
                       ) -> Tuple[Response, str]:
        """
        This interface is used to query the validity of a VAT number. The data is from the Online Invoice Platform of NAV, the Hungarian National Tax and Customs Administration.

        The response always matches the QueryTaxPayerResponse type of Online Invoice Platform of NAV, the Hungarian National Tax and Customs Administration.
        xmlns="http://schemas.nav.gov.hu/OSA/2.0/api"
        xmlns:ns2="http://schemas.nav.gov.hu/OSA/2.0/data"

        https://docs.szamlazz.hu/#querying-taxpayers
        :param vat_number: [str] VAT Number of the queried company
        :return: Tuple[requests.models.Response, requests.models.Response.text]: (Response, returned XML string)
        """
        settings = self.get_basic_settings()
        payload = {
            "vat_number": vat_number,
            **settings,
        }
        r = self.request_maker(
            action="action-szamla_agent_taxpayer",
            template=templates.tax_payer,
            template_data=payload,
            xsd_xml=xsd.tax_payer,
        )
        return r, r.text

    def self_bill(self):
        raise NotImplementedError

    def request_maker(self, action: str, template: str, template_data: dict, xsd_xml: str = "", payload_extra_attachments: dict = None) -> Response:
        """
        Custom, non-managed requests can be made against SzámlaAgent.
        :param action: eg.: action-xmlagentxmlfile
        :param template: a Jinja2 compatible template XML string
        :param template_data: (dict) Data injected into the Jinja2 compatible template XML template
        :param xsd_xml: [optional] The XSD Scheme for XSD scheme compliance check
        :param payload_extra_attachments: (dict) Extra data injected into the Jinja2 compatible template XML template
        :return: requests.models.Response

        Note: Use SzamlazzClient.get_basic_settings() as a skeleton while creating your own template_data. See `get_basic_settings` to learn what fields are available automatically
        """
        t = Template(template)
        output = t.render(template_data)
        logger.debug(f"request_maker / action: {action}")
        logger.debug(f"request_maker / template: {template}")
        logger.debug(f"request_maker / template_data: {template_data}")
        logger.debug(f"request_maker / xsd_xml: {xsd_xml}")
        logger.debug(f"request_maker / Rendered Template Output: {output}")

        if xsd_xml != "":
            ok, err = xsd.validate(xml=output, xsd=xsd_xml)
            if not ok:
                raise xsd.ValidationError(f"XML validation failed: " + err)

        payload = {action: output}
        payload.update(payload_extra_attachments) if payload_extra_attachments else None
        return requests.post(self.url, files=payload)

    def get_basic_settings(self) -> dict:
        """
        get_basic_settings returns a dict containing the following key/value pairs:
          * felhasznalo
          * jelszo
          * szamlaagentkulcs
          * eszamla
          * szamlaLetoltes
          * valaszVerzio

        The values of these keys are dynamically taken from SzamlazzClient.
        :return: dict
        """
        return {
            "felhasznalo": self.username,
            "jelszo": self.password,
            "szamlaagentkulcs": self.agent_key,
            "eszamla": True,
            "szamlaLetoltes": True,
            "valaszVerzio": self.response_version,
        }



import base64
import logging
from pathlib import Path
from requests.models import Response
from typing import NamedTuple, Tuple
from urllib.parse import unquote
# noinspection PyPep8Naming
import xml.etree.ElementTree as ET


__all__ = ["Header", "Merchant", "Buyer", "Item", "Disbursement", "SzamlazzResponse", "PdfDataMissingError", "EmailDetails", ]  # "WayBill"
logger = logging.getLogger(__name__)


class PdfDataMissingError(Exception):
    pass


class Header(NamedTuple):
    """<fejlec>"""
    creating_date: str = ""  # <keltDatum>2020-01-20</keltDatum>
    payment_date: str = ""  # <teljesitesDatum>2020-01-20</teljesitesDatum>
    due_date: str = ""  # <fizetesiHataridoDatum>2020-01-20</fizetesiHataridoDatum>
    payment_type: str = "Átutalás"  # <fizmod>Átutalás</fizmod>
    currency: str = "HUF"  # <penznem>HUF</penznem>
    invoice_language: str = "hu"  # <szamlaNyelve>hu</szamlaNyelve> // can  be: de, en, it, hu, fr, ro, sk, hr
    invoice_comment: str = ""  # <megjegyzes>Invoice comment</megjegyzes>
    name_of_bank: str = "MNB"  # <arfolyamBank>MNB</arfolyamBank>
    exchange_rate: float = 0.0  # <arfolyam>0.0</arfolyam>
    order_number: str = ""  # <rendelesSzam></rendelesSzam>
    pro_forma_number_ref: str = ""  # <dijbekeroSzamlaszam></dijbekeroSzamlaszam>
    deposit_invoice: bool = False  # <elolegszamla>false</elolegszamla>
    invoice_after_deposit_invoice: bool = False  # <vegszamla>false</vegszamla>
    down_payment_invoice_number: str = "" # <elolegSzamlaszam></elolegSzamlaszam>
    correction_invoice: bool = False  # <helyesbitoszamla>false</helyesbitoszamla>
    number_of_corrected_invoice: str = ""  # <helyesbitettSzamlaszam></helyesbitettSzamlaszam>
    proforma_invoice: bool = False  # <dijbekero>false</dijbekero>
    # szallitolevel
    # logoExtra
    invoice_prefix: str = ""  # <szamlaszamElotag></szamlaszamElotag>
    # fizetendoKorrekcio
    # fizetve
    # arresAfa
    # eusAfa
    # szamlaSablon
    # elonezetpdf
    invoice_number: str = ""  # <szamlaszam>E-TST-2011-1</szamlaszam>  // needed for reverse_invoice|storno only
    invoice_template: str = ""  # <!-- Codomain: 'SzlaMost' | 'SzlaAlap' | 'SzlaNoEnv' | 'Szla8cm' | 'SzlaTomb' | 'SzlaFuvarlevelesAlap' -->


class Merchant(NamedTuple):
    """<elado>"""
    bank_name: str = ""  # <bank>BB</bank>
    bank_account_number: str = ""  # <bankszamlaszam>11111111-22222222-33333333</bankszamlaszam>
    reply_email_address: str = ""  # <emailReplyto> </emailReplyto>
    email_subject: str = ""  # <emailTargy>Invoice notification</emailTargy>
    email_text: str = ""  # <emailSzoveg>mail text</emailSzoveg>
    signatory_name: str = "" # <alairoNeve>Kovacs Peter</alairoNeve>


class Buyer(NamedTuple):
    """<vevo>"""
    name: str = ""  # <nev>Kovacs Bt.</nev>
    country: str = ""  # <orszag>Hungary</orszag>
    zip_code: str = ""  # <irsz>2030</irsz>
    city: str = ""  # <telepules>Érd</telepules>
    address: str = ""  # <cim>Tárnoki út 23.</cim>
    email: str = ""  # <email>buyer@example.com</email>
    send_email: bool = False  # <sendEmail>false</sendEmail>
    tax_subject: int = "" # <adoalany>-1</adoalany> <!-- Possible values are 7:business is based outside of the European Union, 6:business is based in the Europen Union, 1: has a hungarian tax number, 0: we don't know, if the buyer has a tax number, -1: no tax number -->
    tax_number: str = ""  # <adoszam>12345678-1-42</adoszam>
    group_id: str = "" # <csoportazonosito>?</csoportazonosito>
    tax_number_eu: str = ""  # <adoszamEU>HU55555555</adoszamEU>
    delivery_name: str = ""  # <postazasiNev>Kovács Bt. mailing name</postazasiNev>
    delivery_country: str = ""  # <postazasiOrszag>Hungary</postazasiOrszag>
    delivery_zip: str = ""  # <postazasiIrsz>2040</postazasiIrsz>
    delivery_city: str = ""  # <postazasiTelepules>Budaörs</postazasiTelepules>
    delivery_address: str = ""  # <postazasiCim>Szivárvány utca 8.</postazasiCim>
    # vevoFokonyv TODO: to be implemented if needed
    identification: str = ""  # <azonosito>1234</azonosito>
    signatory_name: str = "" # <alairoNeve>Kovacs Peter</alairoNeve>
    phone_number: str = ""  # <telefonszam>Tel:+3630-555-55-55, Fax:+3623-555-555</telefonszam>
    comment: str = ""  # <megjegyzes>Call extension 214 from the reception</megjegyzes>


# class WayBill(NamedTuple):
#     """<fuvarlevel>"""
#      <!-- waybill/confinement note, you do not need this: omit the entire tag -->
#     uticel: str = ""  #
#     futarSzolgalat: str = ""  #


class ItemLedger(NamedTuple):
    # language=XML
    """
    <sequence>
        <element name="gazdasagiEsem" type="string" maxOccurs="1" minOccurs="0"></element>
        <element name="gazdasagiEsemAfa" type="string" maxOccurs="1" minOccurs="0"></element>
        <element name="arbevetelFokonyviSzam" type="string" maxOccurs="1" minOccurs="0"></element>
        <element name="afaFokonyviSzam" type="string" maxOccurs="1" minOccurs="0"></element>
        <element name="elszDatumTol" type="date" maxOccurs="1" minOccurs="0"></element>
        <element name="elszDatumIg" type="date" maxOccurs="1" minOccurs="0"></element>
    </sequence>
    """
    economic_event: str = ""  # <gazdasagiesemeny></gazdasagiesemeny>
    economic_event_tax: str = ""  # <gazdasagiesemenyafa></gazdasagiesemenyafa>
    sales_ledger_number: str = ""
    vat_ledger_number: str = ""
    settlement_date_from: str = ""
    settlement_date_to: str = ""


class Item(NamedTuple):
    name: str = ""  # <megnevezes>Elado izé</megnevezes>
    identifier: str = ""  # <azonosito>ASD-123</azonosito>
    quantity: str = ""  # <mennyiseg>1.0</mennyiseg>
    quantity_unit: str = ""  # <mennyisegiEgyseg>db</mennyisegiEgyseg>
    unit_price: str = ""  # <nettoEgysegar>10000</nettoEgysegar>
    vat_rate: str = ""  # <afakulcs>27</afakulcs>
    margin_tax_base: float = ""  # <arresAfaAlap>10.25</arresAfaAlap>
    net_price: str = ""  # <nettoErtek>10000.0</nettoErtek>
    vat_amount: str = ""  # <afaErtek>2700.0</afaErtek>
    gross_amount: str = ""  # <bruttoErtek>12700.0</bruttoErtek>
    comment_for_item: str = ""  # <megjegyzes>lorem ipsum</megjegyzes>
    item_ledger: ItemLedger = ""  # <element name="tetelFokonyv" type="tns:tetelFokonyvTipus" maxOccurs="1" minOccurs="0"></element>


class Disbursement(NamedTuple):
    date: str
    title: str
    amount: float
    description: str = ""


class EmailDetails(NamedTuple):
    addresses: str
    reply_to_address: str
    subject: str
    body_text: str = ""


class SzamlazzResponse:
    def __init__(self,
                 response: Response,
                 xml_namespace: str,
                 ):
        self.xml_namespace = xml_namespace
        self.__response = response
        self.__action_success: bool = False
        content_type = response.headers.get("Content-Type")
        if content_type == "application/octet-stream":
            # Parse XML and map into class members
            root = ET.fromstring(self.__response.text)
            self.__pdf: str = self.__get_tag_text(root, "pdf")
            self.__pdf_bytes: bytes = b""
            self.__action_success: bool = True if (self.__get_tag_text(root, "sikeres") == "true") else False
        else:
            self.__pdf_bytes: bytes = response.content
            self.__pdf: str = base64.b64encode(self.__pdf_bytes).decode("ascii")

        # Error Handling
        self.error_code: str = response.headers.get("szlahu_error_code")
        self.error_message: str = response.headers.get("szlahu_error")
        if self.error_message:
            self.error_message = unquote(self.error_message)
        self.http_request_success: str = "false" if self.error_code else "true"

        # Extract Details
        self.invoice_number: str = response.headers.get("szlahu_szamlaszam")
        self.invoice_net_price: str = response.headers.get("szlahu_nettovegosszeg")
        self.invoice_gross_price: str = response.headers.get("szlahu_bruttovegosszeg")
        self.receivables: str = response.headers.get("szlahu_kintlevoseg")
        self.buyer_account_url: str = response.headers.get("szlahu_vevoifiokurl")
        if self.buyer_account_url:
            self.buyer_account_url = unquote(response.headers.get("szlahu_vevoifiokurl"))
        self.payment_method: str = response.headers.get("szlahu_fizetesmod")

        self.__has_errors = self.error_code or self.error_message
        if self.has_errors:
            logger.error(f"Error Code: {self.error_code}")
            logger.error(f"Error Message: {self.error_message}")

    @property
    def action_success(self) -> bool:
        return self.__action_success

    @property
    def has_errors(self):
        return self.__has_errors

    @property
    def ok(self):
        """
        Shortcut to the original response's attribute with the same name
        """
        return self.__response.ok

    @property
    def response(self) -> Response:
        """
        Original HTTP Response object returned by the requests package
        :return: requests.models.Response
        """
        return self.__response

    @property
    def text(self) -> str:
        """
        Shortcut to the original response's attribute with the same name
        """
        return self.__response.text

    def get_pdf_base64(self) -> str:
        """
        Get PDF from response in Base64 format
        :return: PDF (in Base64 format)
        :rtype: str
        """
        if (not self.__pdf) and (not self.__pdf_bytes):
            raise PdfDataMissingError("No PDF was returned. Check the value of szamlaLetoltes|invoice_download")
        return self.__pdf

    def get_pdf_bytes(self) -> bytes:
        pdf_base64 = self.get_pdf_base64()
        return base64.b64decode(pdf_base64) if pdf_base64 else self.__pdf_bytes

    def write_pdf_to_disk(self, pdf_output_path: Path):
        if not pdf_output_path.parent.exists():
            raise FileNotFoundError(f"Output file's parent folder is missing: {pdf_output_path.parent.as_posix()}")
        data = self.get_pdf_bytes()
        with open(pdf_output_path, "wb+") as f:
            f.write(data)

    def print_errors(self) -> Tuple[str, str]:
        """
        Prints the returned error_code and error_message
        :return: Tuple[error_code, error_message]
        """
        if self.has_errors:
            print("error_code:", self.error_code)
            print("error_message:", self.error_message)
        return self.error_code, self.error_message

    def __get_tag_text(self, root: ET.Element, tag_name):
        tag = root.find(f"{self.xml_namespace}{tag_name}")
        return tag.text if tag is not None else None
